#include "main_window.hpp"
#include "version_window.hpp"
#include "memcard/memorycard.hpp"
#include "make_nana_animation.hpp"
#include "memcard/make_memory_card.hpp"
#include "widget/block_grid.hpp"
#include <nana/gui/filebox.hpp>
#include <iostream>
#include <cassert>

namespace duck {
MainWindow::MainWindow (int icon_size, int icon_fps, const std::string& caption) :
	nana::form(nana::API::make_center(800, 600)),
	m_menubar(*this),
	m_icon_size(icon_size),
	m_icon_fps(icon_fps)
{
	this->caption(caption);
	this->init_menu();

	this->div("vert <menu weight=28><fit margin=3 <fit slots gap=25>>");
	(*this)["menu"] << m_menubar;
	this->collocate();
}

MainWindow::~MainWindow() noexcept = default;

void MainWindow::init_menu() {
	m_menubar.push_back("&File");
	m_menubar.at(0).append("&Open...", [this](nana::menu::item_proxy&){this->open_file_from_dialog();});
	m_menubar.at(0).append("E&xit", [](nana::menu::item_proxy&){nana::API::exit();});

	m_menubar.push_back("&Help");
	m_menubar.at(1).append("Version info", [this](nana::menu::item_proxy&) {
		VersionWindow win(*this);
		win.show();
		win.modality();
	});
}

void MainWindow::open_file_from_dialog() {
	nana::filebox fb(*this, true);
	fb.allow_multi_select(true);
	fb.add_filter(("PSX MemoryCard"), ("*.mcr;*.sav;*.srm"));
	fb.add_filter(("All files"), ("*.*"));
	auto files = fb();

	for (const auto& file : files) {
		m_memcards.push_back(mc::psx::make_memory_card(file));
		m_grids.emplace_back(*this, 3, true);
		m_grid_labels.emplace_back(*this, std::string("Slot ") + std::to_string(m_grid_labels.size() + 1));
#if !defined(NDEBUG)
		m_grids.back().bgcolor(nana::colors::azure);
#endif

		auto& mc = m_memcards.back();
		auto& grid = m_grids.back();
		for (int z = 0; z < 15; ++z) {
			auto block = mc[z];
			for (std::size_t n = 0; block.has_magic() and n < block.block_count(); ++n) {
				grid.emplace_back(duck::make_nana_animation(block, m_icon_size, m_icon_size, m_icon_fps));
			}
		}
		(*this)["slots"] << m_grids.back();
	}
	this->collocate();
}
} //namespace duck
